﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Invoice.Payed.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.Invoice',
    xtype: 'App.Control.Kivii.Finances.Invoice.Payed.List',
    // dataSource: '/Restful/Kivii.Finances.Entities.Invoice/QueryEx.json',
    requires: ['App.Control.DownloadField'],
    title: '发票到账',
    iconCls: 'x-fa fa-money',
    // filter: {
    //     BeginTime: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
    //     EndTime: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
    //     // Debited: false,
    //     OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid
    // },
    //在末尾保留足够的滚动条大小的控件
    reserveScrollbar:true,
    //默认情况下启用缓冲渲染
    //启用缓存会造成空白行的问题,有些数据过多会预留渲染控件,切换成数据少的会预留空白
    bufferedRenderer:false,
    viewModel: {
        data: {
            _selection: null,
            paymentKvid: null,
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 400,
        sorters: [{
            property: 'OperateTime',
            direction: 'DESC'
        }]
    },
    plugins: [{
        ptype: 'cellediting',
        clicksToEdit: 2,
    }],
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
                xtype: 'searchField',
                flex: 1,
                minWidth: 250,
                labelWidth: 60,
                queryField: 'SerialNumber,PayerName,OperatorName',
                emptyText: '模糊搜索流水号、付款单位、申请人',
            }, {
                xtype: 'downloadField',
                dataSource: '/Restful/Kivii.Finances.Entities.Invoice/Query.xlsx', //数据源
                fileName: '数据导出',
                window: {
                    type: 'App.Control.Kivii.Finances.General.Template.Selector',
                    title: '选择模板',
                    iconCls: 'x-fa fa-file-excel-o',
                    autoScroll: true,
                    width: 700,
                    height: 350,
                    closeAction: 'hide',
                    modal: true,
                    layout: 'fit',
                    filter: {
                        FolderPath: '/Finances/Invoice/Files',
                        TagsContains: 'QueryEx'
                    }
                }
            }
        ]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            text: '发票编码',
            dataIndex: 'SerialNumber',
            width: 90
        }, {
            text: '所属部门',
            dataIndex: 'OwnerName',
            width: 100
        }, {
            text: '发票抬头',
            dataIndex: 'PayerName',
            renderer: '_renderPayerName',
            width: 200
        }, {
            text: '金额(含税)',
            align: 'right',
            dataIndex: 'Amount',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 110
        }, {
            text: '税额',
            align: 'right',
            dataIndex: 'AmountTax',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 100
        }, {
            text: '金额(不含税)',
            align: 'right',
            dataIndex: 'AmountUntaxed',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 120
        }, {
            xtype: 'datecolumn',
            text: '开票日期',
            format: 'Y-m-d',
            dataIndex: 'OperateTime',
            width: 100
        }, {
            text: '收款金额',
            align: 'right',
            dataIndex: 'AmountPayment',
            renderer: '_renderAmount',
            width: 110
        }, {
            xtype: 'datecolumn',
            text: '收款日期',
            format: 'Y-m-d',
            dataIndex: 'PayedTime',
            editor: {
                xtype: 'datefield',
                altFormats: 'D, d M Y H:i:s T|c|Y-m-d|Y-m-d H:i|Y/n/j'//编辑时需要加上这段format 不然原数据不会显示出来
            },
            width: 100
        }, {
            text: '申请人',
            dataIndex: 'OperatorName',
            renderer: '_renderOperatorName',
            width: 140
        },  {
            text: '操作人',
            dataIndex: 'CreatorName',
            width: 100
        },{
            text: '类型',
            dataIndex: 'Type',
            width: 130
        }, {
            text: '分类',
            dataIndex: 'Category',
            renderer: '_renderCategory',
            width: 90
        }, {
            text: '税号',
            dataIndex: 'PayerTaxNumber',
            width: 200
        }, {
            text: '地址',
            dataIndex: 'PayerCompanyAddress',
            renderer: '_renderPayerCompanyAddress',
            width: 260
        }, {
            text: '开户行',
            dataIndex: 'PayerRegisteredBank',
            renderer: '_renderPayerRegisteredBank',
            width: 260
        }, {
            text: '备注',
            dataIndex: 'Remark',
            minWidth: 180,
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate'
    },
    setOwner: function(_owner, _ctrl) {
        let _view = this;
        if (_view.filter === null) _view.setFilter({
        });
        _view.filter.Type = "Relation";
        _view.filter.OffsetKvid='00000000-0000-0000-0000-000000000000';
        _view.filter.ParentKvid = _owner.Kvid;
        _view.reload();
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _renderAmount: function(_value, _meta, _record, _rowIndex) {
            if (_value === _record.data.Amount) {
                _meta.style = 'background-color:#B4EEB4';
            }
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        _renderOperatorName: function(_value, _meta, _record, _rowIndex) {
            if (_record.data.hasOwnProperty('OwnerName')) return _value + '(' + _record.data.OwnerName + ')';
            return _value;
        },
        _renderPayerName: function(_value, _meta, _record, _rowIndex) {
            if (_record.data.hasOwnProperty('OffsetKvid') && _record.data.OffsetKvid !== '00000000-0000-0000-0000-000000000000') {
                _meta.style = 'background-color:#FFC0CB';
                return _value + '[已作废]';
            }
            return _value;
        },
        _renderCategory: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Debit") {
                _meta.style = 'background-color:#F3F781';
                return "借票";
            }
            if (_value === "Payment") {
                _meta.style = 'background-color:#B4EEB4';
                return "到账";
            }
            return _value;
        },
        _renderPayerCompanyAddress: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerPhone;
        },
        _renderPayerRegisteredBank: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerBankAccount;
        }
    }
});