﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Payment.Import.Excel.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.Payment',
    xtype: 'App.Control.Kivii.Finances.Payment.Import.Excel.List',
    viewModel: {
        data: {
            _selection: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: false,
        autoSync: false,
        pageSize: 10000
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    features: [{
        ftype: 'summary'
    }],
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: ['->', {
            text: '导入',
            iconCls: 'x-fa fa-upload',
            handler: '_onCreate'
        }]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            xtype: 'actioncolumn',
            width: 80,
            text: '操作',
            align: 'center',
            menuDisabled: true,
            sortable: false,
            items: [{
            	text:'编辑',
                iconCls: 'x-fa fa-edit',
                handler: '_onEditDetail'
            }, '->',{
            	text:'删除',
                iconCls: 'x-fa fa-trash-o',
                handler: '_onDeleteDetail'
            }]
            },{
                text: '流水号',
                dataIndex: 'SerialNumber',
                width: 120
        }, {
            text: '所属部门',
            dataIndex: 'OwnerName',
            width: 120
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return '合计 ￥';
            },
            width: 220
        }, {
            text: '发生金额',
            align: 'right',
            dataIndex: 'Amount',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 110
        }, {
            text: '付款类型',
            dataIndex: 'Type',
            renderer: '_renderType',
            width: 100
        }, {
            xtype: 'datecolumn',
            text: '发生日期',
            format: 'Y-m-d H:i',
            dataIndex: 'OperateTime',
            width: 150
        }, {
            text: '操作人',
            dataIndex: 'OperatorName',
            width: 100
        }, {
            text: '摘要',
            dataIndex: 'Summary',
            width: 150
        }, {
            text: '付款银行',
            dataIndex: 'PayerAccountName',
            width: 280
        }, {
            text: '付款账号',
            dataIndex: 'PayerAccountSerialNumber',
            width: 180
        }, {
            text: '备注',
            dataIndex: 'Remark',
            flex: 1
        }]
    },
    setOwner: function(_entities) {
        let _view = this;
        let _store = _view.getStore();
        _store.add(_entities);
    },
    controller: {
        _renderAmount: function(_value, _meta, _record, _rowIndex) {
            if (_value === _record.data.Amount) {
                _meta.style = 'background-color:#B4EEB4';
            }
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === 'Cash') {
                return '现金';
            }
            if (_value === 'Pos') {
                return '刷卡';
            }
            if (_value === 'AliPay') {
                return '支付宝';
            }
            if (_value === 'WeChat') {
                return '微信';
            }
            if (_value === 'Bank') {
                return '银行转账';
            }
            return _value;
        },
        _onDeleteDetail: function(_ctrl, _rowIndex, _colIndex, _item, _e, _record) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            _view.removeEntity(_record.data);
        },
        _onEditDetail: function(_ctrl, _rowIndex, _colIndex, _item, _e, _record) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            
            let _form = Ext.create('App.Control.Kivii.Finances.Payment.Import.Excel.Edit.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                // header:false,
                iconAlign: 'left',
                autoScroll: true,
                width: 900,
                height: 550,
                modal: true,
                resizable: false,
                items: [_form]
            });
            _win.show();
            var _data={};
            Ext.apply(_data,_record.data);
            _form.setOwner(_data,_item);
            _form.on('entityModify', function(_ctrl, _entity) {
                _win.close();
                _view.modifyEntity(_entity);
            });
        },


        _onCreate: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _store=_view.getStore();
            let _data = _store.getData().items;
            if (_data.length<=0) return;
            let _msg = Ext.create('Ext.window.MessageBox', {
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                closeAction: 'destroy'
            });
            _msg.confirm(_ctrl.text, '确认导入流水?\n重复项会被忽略!', function(_optional){
            	if (_optional !== 'yes') return;
            	_controller._acceptData(_ctrl);
            }, _controller);
            
        },
        _acceptData: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _store=_view.getStore();
            let _data = _store.getData().items;
            if (_data.length<=0) return;
            let _jsonData={};
            _jsonData.Items=[];
            for (let _i = 0; _i < _data.length; _i++) {
            	let _item=_data[_i].data;
            	_jsonData.Items.push(_item);
            }
            if(_jsonData.Items.length<=0)return;
            _ctrl.setDisabled(true);
            Ext.Ajax.request({
                url: '/Restful/' + _view.entityType + '/Accept.json',
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    if(_rtns.Results.length<=0) Ext.toast("暂无新增数据,已跳过重复项!");
                    else Ext.toast("新增成功,并跳过重复项!");
                    _view.fireEvent('entityCreated', _view, _rtns.Results);
                    _ctrl.setDisabled(false);
                },
                failure: function(_response) {
                    Ext.toast("新增失败!");
                    _ctrl.setDisabled(false);
                }
            });
        }
    }
});