﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    [Api(Description = "内部付款,用于内部账户之间转账使用,区分开实际银行等方式打款来的Payment表")]
    [Alias(Configs.TableNameTransfer)]
    public class Transfer:EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasSummary, IEntityHasRemark,
        IEntityHasOffset, IEntityHasBiz,
        IEntityHasCreator, IEntityHasUpdater, IEntityHasOrganization
    {
        #region 关联信息 Owner,Offset Biz
        /// <summary>
        /// 一般不使用此字段，预留
        /// </summary>
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OwnerKvid { get; set; }

        [InternalSetter]
        [StringLength(500)]
        public string OwnerName { get; set; }
        /// <summary>
        /// 冲帐关联
        /// </summary>
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OffsetKvid { get; set; }

        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [InternalSetter]
        [StringLength(200)]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型")]
        [InternalSetter]
        [StringLength(100)]
        public string BizType { get; set; }
        #endregion

        #region 收付双方 Payer付款单位,贷方  Payee收款单位,借方
        [IgnoreUpdate]
        [DefaultEmptyGuid, Required]
        public Guid PayerKvid { get; set; }

        [ApiMember(Description = "付款单位,贷方")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayerName { get; set; }


        [IgnoreUpdate]
        [DefaultEmptyGuid, Required]
        public Guid PayeeKvid { get; set; }

        [ApiMember(Description = "收款单位,借方")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayeeName { get; set; }
        #endregion

        #region 收付款账号相关
        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayerAccountKvid { get; set; }


        [ApiMember(Description = "付款账户")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayerAccountOwnerName { get; set; }

        [ApiMember(Description = "付款账户名称")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayerAccountName { get; set; }

        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayerAccountSerialNumber { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayeeAccountKvid { get; set; }

        [ApiMember(Description = "收款账户")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayeeAccountOwnerName { get; set; }

        [ApiMember(Description = "收款账户名称")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayeeAccountName { get; set; }

        [ApiMember(Description = "收款账户卡号")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayeeAccountSerialNumber { get; set; }
        #endregion

        #region 操作相关  Organization Creator Updater Operator OperateTime 

        [ApiMember(Description = "操作日期")]
        [IgnoreUpdate]
        [Required]
        public DateTime OperateTime { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        [ApiMember(Description = "经 办 人")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string OperatorName { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid AuditorKvid { get; set; }

        [ApiMember(Description = "审核人")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string AuditorName { get; set; }


        [ApiMember(Description = "组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }

        #endregion

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        [IgnoreUpdate]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大2000字")]
        [StringLength(2000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 收付款相关
        [ApiMember(Description = "流水号")]
        [IgnoreUpdate]
        [StringLength(100), Default(""), SerialNumber]
        public string SerialNumber { get; set; }

        [ApiMember(Description = "付款类型,现金、银行汇款等")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public TransferType Type { get; set; }

        [ApiMember(Description = "发生金额")]
        [IgnoreUpdate]
        [DecimalLength(15, 2), Default(0)]
        public decimal Amount { get; set; }

        [ApiMember(Description = "单位")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }

        #endregion
    }

    public enum TransferType
    {
        Unsupported = 0,
        /// <summary>
        /// 现金业务，充值和提现
        /// </summary>
        Deposit,

        /// <summary>
        /// 内部账号转账
        /// </summary>
        Transfer,

        /// <summary>
        /// 合同付款业务
        /// </summary>
        Contract,

        /// <summary>
        /// 贷款业务
        /// </summary>
        Loan,

        /// <summary>
        /// 其它类型的业务转账
        /// </summary>
        Biz
    }
}
