﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances
{
    public static class PaymentExtension
    {
        /// <summary>
        /// complate!
        /// 获取子付款记录的流水号
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static string GetSubSerialNumber(this Payment payment, IDbConnection conn = null)
        {
            payment.RootKvid.ThrowIfEmpty("RootKvid不能为空");
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Payment>();
            //var rootPayment = conn.SingleById<Payment>(payment.RootKvid);
            //rootPayment.ThrowIfNull("主付款记录不存在");
            var total = conn.Count<Payment>(o => o.RootKvid == payment.RootKvid);
            if (total == 0) return string.Empty;
            //allcount不需要加1，因为Root的也统计进去了
            return $"{payment.SerialNumber}-{total}";
        }

        /// <summary>
        /// complate!
        /// 接受一个从系统外来的新付款记录
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static AccountDetail Accept(this Payment payment, out Pay pay)
        {
            payment.ThrowIfNull("付款记录不能为空!");
            (payment.Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位");
            (payment.PayerAccountKvid == Guid.Empty && payment.PayeeAccountKvid == Guid.Empty).ThrowIfTrue("收付款账户不能同时为空!");
            (payment.PayerAccountKvid != Guid.Empty && payment.PayeeAccountKvid != Guid.Empty).ThrowIfTrue("收付款账户不能同时指定!");
            (payment.Amount <= 0).ThrowIfTrue("付款记录发生金额不能少于0!");
            (payment.AmountInvoice == 0).ThrowIfFalse("付款记录开票金额不为0!");
            (payment.AmountUsed == 0).ThrowIfFalse("付款记录使用金额不为0!");
            (payment.Type != PaymentType.Cash && payment.Type != PaymentType.Pos && payment.Type != PaymentType.Bank).ThrowIfTrue("仅支持接受现金刷卡或者银行流水记录");
            if (payment.Type == PaymentType.Bank) (payment.AmountSplited != 0).ThrowIfTrue("付款记录拆分不为0!");
            else (payment.AmountSplited != payment.Amount).ThrowIfTrue("现金刷卡拆分金额错误!");
            if (payment.ParentKvid != Guid.Empty) payment.ParentKvid = Guid.Empty;
            if (payment.OffsetKvid != Guid.Empty) payment.OffsetKvid = Guid.Empty;
            if (payment.Kvid == Guid.Empty) payment.Kvid = Guid.NewGuid();
            if (payment.RootKvid != payment.Kvid) payment.RootKvid = payment.Kvid;


            #region 付款账户变动
            var rtns = new AccountDetail();
            rtns.AccountKvid = payment.PayerAccountKvid;
            rtns.BizId = payment.SerialNumber;
            rtns.BizKvid = payment.Kvid;
            rtns.BizType = typeof(Payment).FullName;

            rtns.PayerKvid = payment.PayerKvid;
            rtns.PayerName = payment.PayerName;
            rtns.PayerAccountKvid = payment.PayerAccountKvid;
            rtns.PayerAccountName = payment.PayerAccountName;
            rtns.PayerAccountSerialNumber = payment.PayerAccountSerialNumber;

            rtns.PayeeKvid = payment.PayeeKvid;
            rtns.PayeeName = payment.PayeeName;
            rtns.PayeeAccountKvid = Guid.Empty;//接受外部的钱到payment,所以收款账户为空
            rtns.PayeeAccountName = payment.PayeeAccountName;
            rtns.PayeeAccountSerialNumber = payment.PayeeAccountSerialNumber;

            rtns.AmountPayment = payment.Amount;

            rtns.Summary = payment.Summary;
            rtns.Summary = payment.Summary;
            rtns.Remark = payment.Remark;
            #endregion

            pay = new Pay();
            pay.OwnerKvid = payment.OwnerKvid;//KiviiContext.CurrentMember.DepartmentKvid;
            pay.OwnerName = payment.OwnerName;//KiviiContext.CurrentMember.DepartmentName;
            pay.PayerKvid = payment.PayerKvid;
            pay.PayerName = payment.PayerName;
            pay.FromType = PayType.Account;//付款来源记录 是账户还是Payment
            pay.FromKvid = payment.PayerAccountKvid;
            pay.PayeeKvid = payment.PayeeKvid;
            pay.PayeeName = payment.PayeeName;
            pay.ToType = PayType.Payment;//收款来源记录
            pay.ToKvid = payment.Kvid;
            pay.Type = "Accept";
            pay.PayedTime = DateTime.Now;
            pay.Amount = payment.Amount;
            pay.AmountPayment = pay.Amount;
            pay.AmountDiscount = 0;
            pay.Currency = payment.Currency;
            pay.OperateTime = DateTime.Now;
            pay.OperatorName = KiviiContext.CurrentMember.FullName;
            pay.OperatorKvid = KiviiContext.CurrentMember.Kvid;

            return rtns;
        }

        /// <summary>
        /// complate!
        /// 到账转账至账户
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="accountPayee"></param>
        /// <param name="amount"></param>
        /// <param name="remark"></param>
        /// <param name="pay"></param>
        /// <param name="accountDetail"></param>
        /// <returns></returns>
        public static Payment Transfer(this Payment payment, Account accountPayee, decimal amount, string remark, out Pay pay, out AccountDetail accountDetail)
        {
            payment.ThrowIfNull("到账不能为空!");
            accountPayee.ThrowIfNull("账户不能为空!");
            (amount <= 0).ThrowIfTrue("入账金额不能小于等0!");
            (payment.OffsetKvid != Guid.Empty).ThrowIfTrue("此项已冲账!");
            if (payment.Amount - payment.AmountUsed < amount) throw new Exception($"此到账可使用金额不足,仅剩:{payment.Amount - payment.AmountUsed}元");
            (payment.Currency != accountPayee.Currency).ThrowIfTrue("账户的货币单位不一致!");

            var transferPaymentKvid = Guid.NewGuid();
            #region 创建入账Payment
            var rtns = new Payment();
            rtns.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            rtns.OwnerName = KiviiContext.CurrentMember.DepartmentName;
            rtns.Kvid = transferPaymentKvid;
            rtns.SerialNumber = payment.SerialNumber;
            rtns.RootKvid = payment.RootKvid;
            rtns.ParentKvid = payment.Kvid;
            rtns.Type = PaymentType.Transfer;
            rtns.BizId = accountPayee.Name;
            rtns.BizKvid = accountPayee.Kvid;
            rtns.BizType = typeof(Account).FullName;
            rtns.PayerName = payment.PayerName;
            rtns.PayerKvid = payment.PayerKvid;
            //rtns.PayerAccountName = accountPayer.Name;
            //rtns.PayerAccountKvid = accountPayer.Kvid;
            //rtns.PayerAccountOwnerName = accountPayer.OwnerName;
            //rtns.PayerAccountSerialNumber = accountPayer.SerialNumber;
            rtns.PayeeName = accountPayee.Name;
            rtns.PayeeKvid = accountPayee.Kvid;
            rtns.PayeeAccountName = accountPayee.Name;
            rtns.PayeeAccountKvid = accountPayee.Kvid;
            rtns.PayeeAccountOwnerName = accountPayee.OwnerName;
            rtns.PayeeAccountSerialNumber = accountPayee.SerialNumber;
            rtns.Amount = amount;
            rtns.AmountSplited = 0;
            rtns.AmountUsed = amount;
            rtns.OperateTime = DateTime.Now;
            rtns.OperatorName = KiviiContext.CurrentMember.FullName;
            rtns.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            rtns.Currency = payment.Currency;
            rtns.Summary = $"{remark}";
            rtns.Remark = $"{remark}";
            #endregion

            //此处不更新父级AmountUsed,因为在事务中 调用RecalculateAmountUsed方法即可
            //payment.AmountUsed += amount;
            //payment.AddOnlyProperties(o => o.AmountSplited);

            #region 收款账户变动
            accountDetail = new AccountDetail();
            accountDetail.AccountKvid = accountPayee.Kvid;
            accountDetail.BizId = payment.SerialNumber;
            accountDetail.BizKvid = payment.Kvid;
            accountDetail.BizType = typeof(Payment).FullName;
            accountDetail.PayerKvid = payment.PayerKvid;
            accountDetail.PayerName = payment.PayerName;
            //rtns.PayerAccountKvid = payerAccount.Kvid;
            //rtns.PayerAccountName = payerAccount.Name;
            //rtns.PayerAccountSerialNumber = payerAccount.SerialNumber;
            accountDetail.PayeeKvid = payment.PayeeKvid;
            accountDetail.PayeeName = payment.PayeeName;
            accountDetail.PayeeAccountKvid = accountPayee.Kvid;
            accountDetail.PayeeAccountName = accountPayee.Name;
            accountDetail.PayeeAccountSerialNumber = accountPayee.SerialNumber;
            accountDetail.AmountPayment = amount;
            accountDetail.Summary = payment.Summary;
            accountDetail.Remark = remark;
            #endregion

            pay = new Pay();
            pay.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            pay.OwnerName = KiviiContext.CurrentMember.DepartmentName;
            pay.PayerKvid = rtns.PayerKvid;
            pay.PayerName = rtns.PayerName;
            pay.FromKvid = rtns.Kvid;
            pay.FromType = PayType.Payment;
            pay.PayeeKvid = accountPayee.Kvid;
            pay.PayeeName = accountPayee.Name;
            pay.ToKvid = accountPayee.Kvid;
            pay.ToType = PayType.Account;
            pay.Type = "Transfer";
            pay.PayedTime = DateTime.Now;
            pay.Amount = rtns.Amount;
            pay.AmountPayment = pay.Amount;
            pay.AmountDiscount = 0;
            pay.Currency = rtns.Currency;
            pay.OperateTime = DateTime.Now;
            pay.OperatorName = KiviiContext.CurrentMember.FullName;
            pay.OperatorKvid = KiviiContext.CurrentMember.Kvid;

            return rtns;
        }

        /// <summary>
        /// complate!
        /// 重新计算Payment的 AmountUsed
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Payment RecalculateAmountUsed(this Payment payment, IDbConnection conn = null)
        {
            payment.ThrowIfNull("要更新的付款记录不能为空!");
            if (payment.Type == PaymentType.Unsupported || payment.Type == PaymentType.Transfer || payment.Type == PaymentType.DynamicPay) throw new Exception("不支持的类型!");

            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Payment>();
            }
            decimal amountUsed = 0;
            if (payment.Type == PaymentType.Split)//拆分类型 要统计使用金额就跳过子集assgin,因为此类型也可能存在子集  所以把他们合并起来一起查
            {
                var queryAssign = conn.From<Payment>();
                queryAssign.Where(o => o.ParentKvid == payment.Kvid && o.OffsetKvid == Guid.Empty && o.Type == PaymentType.Assign);
                queryAssign.Select(o => o.Kvid);
                var assignKvids = conn.Select<Guid>(queryAssign);
                if (assignKvids.IsNullOrEmpty()) assignKvids = new List<Guid>();
                assignKvids.Add(payment.Kvid);
                amountUsed = conn.Scalar<Payment, decimal>(o => Sql.Sum(o.AmountUsed), p => p.Type != PaymentType.Assign && Sql.In(p.ParentKvid, assignKvids) && p.OffsetKvid == Guid.Empty);
            }
            else amountUsed = conn.Scalar<Payment, decimal>(o => Sql.Sum(o.AmountUsed), p => p.ParentKvid == payment.Kvid && p.OffsetKvid == Guid.Empty);
            Payment parentPayment = null;
            if (payment.ParentKvid != Guid.Empty) parentPayment = conn.SingleById<Payment>(payment.ParentKvid);
            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction();
            try
            {
                if (payment.OffsetKvid == Guid.Empty)
                {
                    //payment.AmountInvoice = amountInvoiced;
                    //payment.AddOnlyProperties(o => o.AmountInvoice);
                    payment.AmountUsed = amountUsed;
                    payment.AddOnlyProperties(o => o.AmountUsed);
                    conn.UpdateOnly(payment);
                }
                if (parentPayment != null) parentPayment.RecalculateAmountUsed(conn);
                trans?.Commit();
                return payment;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }

        /// <summary>
        /// complate!
        /// 重新计算Payment的 AmountSplit
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Payment RecalculateAmountSplit(this Payment payment, IDbConnection conn = null)
        {
            payment.ThrowIfNull("要更新的付款记录不能为空!");
            (payment.Type != PaymentType.Bank && payment.Type != PaymentType.Split).ThrowIfTrue("不支持的类型");
            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Payment>();
            }

            //var amountInvoiced = conn.Scalar<Payment, decimal>(o => Sql.Sum(o.AmountInvoice), p => p.ParentKvid == payment.Kvid && p.OffsetKvid == Guid.Empty);
            var amountSplit = conn.Scalar<Payment, decimal>(o => Sql.Sum(o.Amount), p => p.ParentKvid == payment.Kvid && Sql.In(p.Type, PaymentType.Assign, PaymentType.Split, PaymentType.UnBiz, PaymentType.Refund) && p.OffsetKvid == Guid.Empty);
            Payment parentPayment = null;
            if (payment.ParentKvid != Guid.Empty) parentPayment = conn.SingleById<Payment>(payment.ParentKvid);
            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction();
            try
            {
                if (payment.OffsetKvid == Guid.Empty)
                {
                    //payment.AmountInvoice = amountInvoiced;
                    //payment.AddOnlyProperties(o => o.AmountInvoice);
                    payment.AmountSplited = amountSplit;
                    payment.AddOnlyProperties(o => o.AmountSplited);
                    conn.UpdateOnly(payment);
                }
                if (parentPayment != null) parentPayment.RecalculateAmountSplit(conn);
                trans?.Commit();
                return payment;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }

        /// <summary>
        /// complate!
        /// 重新计算Payment的 AmountInvoice
        /// 只针对Bank等根级Payment
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Payment RecalculateAmountInvoice(this Payment payment, IDbConnection conn = null)
        {
            payment.ThrowIfNull("要更新的付款记录不能为空!");
            (payment.OffsetKvid != Guid.Empty).ThrowIfTrue("此到账已作废 无法重新计算开票金额");
            (payment.Type != PaymentType.Bank && payment.Type != PaymentType.Split).ThrowIfTrue("不支持的类型");
            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Payment>();

            var amountInvoiced = conn.Scalar<Payment, decimal>(o => Sql.Sum(o.AmountInvoice), p => Sql.In(p.Type, PaymentType.Assign, PaymentType.Split) && p.ParentKvid == payment.Kvid && p.OffsetKvid == Guid.Empty);

            Payment parentPayment = null;
            if (payment.ParentKvid != Guid.Empty) parentPayment = conn.SingleById<Payment>(payment.ParentKvid);
            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction();
            try
            {
                payment.AmountInvoice = amountInvoiced;
                payment.AddOnlyProperties(o => o.AmountInvoice);
                conn.UpdateOnly(payment);
                if (parentPayment != null) parentPayment.RecalculateAmountInvoice(conn);
                trans?.Commit();
                return payment;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }

        /// <summary>
        /// 重新计算Assign类型的AmountInvoice或AmountUsed
        /// 如果是上级到账进行开票,收款登记或充值的话,存在子集Assign则需要向下更新对应的AmountInvoice和AmountUsed
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static List<Payment> RecalculateAssignAmount(this Payment payment, IDbConnection conn = null)
        {
            payment.ThrowIfNull("要更新的付款记录不能为空!");
            (payment.OffsetKvid != Guid.Empty).ThrowIfTrue("此到账已作废 无法重新计算开票金额");
            (payment.Type != PaymentType.Split).ThrowIfTrue("不支持的类型");
            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Payment>();
            //查看已经拆分的项目分得的已经开票金额 
            var existSplits = conn.Select<Payment>(o => o.ParentKvid == payment.Kvid && o.OffsetKvid == Guid.Empty && o.Type == PaymentType.Assign);
            if (existSplits.IsNullOrEmpty()) return null;

            if (payment.AmountInvoice <= 0)
            {
                foreach (var item in existSplits)
                {
                    if (item.AmountInvoice == 0) continue;
                    item.AmountInvoice = 0;
                    item.AddOnlyProperties(o => o.AmountInvoice);
                }
            }
            else //如果要拆分的到账已经开过票 就要计算拆分项的开票金额
            {
                if (payment.AmountInvoice < payment.Amount)//到账的部分进行了开票 就需要计算拆分项的开票金额
                {
                    var amountInvoice = payment.AmountInvoice;
                    foreach (var item in existSplits)
                    {
                        var currentAmount = amountInvoice > item.Amount ? item.Amount : amountInvoice;
                        item.AmountInvoice = currentAmount;
                        item.AddOnlyProperties(o => o.AmountInvoice);
                        amountInvoice -= currentAmount;
                    }
                }
                else
                {
                    foreach (var item in existSplits)
                    {
                        if (item.AmountInvoice == item.Amount) continue;
                        item.AmountInvoice = item.Amount;
                        item.AddOnlyProperties(o => o.AmountInvoice);
                    }

                }

            }

            if (payment.AmountUsed <= 0)
            {
                foreach (var item in existSplits)
                {
                    if (item.AmountUsed == 0) continue;
                    item.AmountUsed = 0;
                    item.AddOnlyProperties(o => o.AmountUsed);
                }
            }
            else //如果要拆分的到账已经开过票 就要计算拆分项的开票金额
            {
                if (payment.AmountUsed < payment.Amount)//到账的部分进行了开票 就需要计算拆分项的开票金额
                {
                    var amountUsed = payment.AmountUsed;
                    foreach (var item in existSplits)
                    {
                        var currentAmount = amountUsed > item.Amount ? item.Amount : amountUsed;
                        item.AmountUsed = currentAmount;
                        item.AddOnlyProperties(o => o.AmountUsed);
                        amountUsed -= currentAmount;
                    }
                }
                else
                {
                    foreach (var item in existSplits)
                    {
                        if (item.AmountUsed == item.Amount) continue;
                        item.AmountUsed = item.Amount;
                        item.AddOnlyProperties(o => o.AmountUsed);
                    }

                }

            }

            var rtns = new List<Payment>();

            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction();
            try
            {
                foreach (var item in existSplits)
                {
                    if (item.OnlyProperties.IsNullOrEmpty()) continue;
                    conn.UpdateOnly(item);
                    rtns.Add(item);
                }
                trans?.Commit();
                return rtns;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }
        /// <summary>
        /// complate!
        /// 到账冲账,此方法不做业务判断,传过来的Payment均可Offset,除已经冲账过的
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="remark"></param>
        /// <returns></returns>
        public static Payment Offset(this Payment payment, string remark)
        {
            payment.ThrowIfNull("付款记录不能为空!");
            (payment.Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位");
            (payment.OffsetKvid != Guid.Empty).ThrowIfTrue("收项已冲账");

            #region payment冲账
            var newPaymentKvid = Guid.NewGuid();
            payment.OffsetKvid = newPaymentKvid;
            payment.AddOnlyProperties(o => o.OffsetKvid);
            payment.Remark += $" {remark}";
            payment.AddOnlyProperties(o => o.Remark);

            //新建一个和原来一样的
            var offsetPayment = new Payment();
            offsetPayment.PopulateWith(payment);
            offsetPayment.OffsetKvid = payment.Kvid;//新的冲帐关联用原来的Kvid
            offsetPayment.Kvid = newPaymentKvid;
            offsetPayment.SerialNumber = payment.SerialNumber.IsNullOrEmpty() ? "" : ("-" + payment.SerialNumber);
            offsetPayment.Remark = $"{remark}";
            offsetPayment.Amount = 0 - offsetPayment.Amount;//反向值
            offsetPayment.OperateTime = DateTime.Now;
            offsetPayment.OperatorName = KiviiContext.CurrentMember.FullName;
            offsetPayment.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            #endregion

            return offsetPayment;
        }

        /// <summary>
        /// complate!
        /// 到账划转,并且更新原payment.AmountSplit字段
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="amountAssign"></param>
        /// <param name="remark"></param>
        /// <param name="ownerKvid"></param>
        /// <param name="ownerName"></param>
        /// <param name="conn"></param>
        /// <param name="AmountFirst">标记划转出去的金额没有开票和没有使用优先,默认true,优先划转已经开票和使用的金额</param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        public static Payment BizAssign(this Payment payment, decimal amountAssign, string remark = null, Guid? ownerKvid = null, string ownerName = null, IDbConnection conn = null, bool AmountFirst = true)
        {
            payment.ThrowIfNull($"传入payment参数为空");
            if (payment.Type != PaymentType.Split) throw new Exception("请选择正确类型!");
            if (payment.OffsetKvid != Guid.Empty) throw new Exception("已冲销收款不可划转!");
            (amountAssign <= 0).ThrowIfTrue("金额不能为小于0。");
            if (payment.AmountSplited == payment.Amount) throw new Exception("无剩余收款可划转!");
            if (payment.AmountSplited + amountAssign > payment.Amount) throw new Exception("划转金额超出范围!");
            if (payment.OwnerKvid == ownerKvid) throw new Exception("不可划转至同部门!");
            decimal amountInvoice = 0;
            decimal amountUsed = 0;
            #region 计算开票金额和使用金额
            if (AmountFirst)
            {
                if (payment.AmountInvoice > 0 || payment.AmountUsed > 0)//如果要拆分的到账已经开过票 就要计算拆分项的开票金额
                {
                    if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Payment>();
                    //查看已经拆分的项目分得的已经开票金额 
                    var existSplits = conn.Select<Payment>(o => o.RootKvid == payment.RootKvid && o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.Assign));
                    if (payment.AmountInvoice < payment.Amount)//到账的部分进行了开票 就需要计算拆分项的开票金额
                    {
                        if (!existSplits.IsNullOrEmpty())
                        {
                            //剩余的开票金额
                            var remainAmountInvoice = payment.AmountInvoice - existSplits.Sum(o => o.AmountInvoice);
                            if (remainAmountInvoice > 0) amountInvoice = remainAmountInvoice > amountAssign ? amountAssign : remainAmountInvoice;
                        }
                        else
                        {
                            amountInvoice = payment.AmountInvoice > amountAssign ? amountAssign : payment.AmountInvoice;
                        }
                    }
                    else amountInvoice = amountAssign;//否则直接记录拆分项完全开票
                    if (payment.AmountUsed < payment.Amount)
                    {
                        if (!existSplits.IsNullOrEmpty())
                        {
                            //剩余的开票金额
                            var remainAmountUsed = payment.AmountUsed - existSplits.Sum(o => o.AmountUsed);
                            if (remainAmountUsed > 0) amountUsed = remainAmountUsed > amountAssign ? amountAssign : remainAmountUsed;
                        }
                        else
                        {
                            amountUsed = payment.AmountUsed > amountAssign ? amountAssign : payment.AmountUsed;
                        }
                    }
                    else amountUsed = amountAssign;
                }
            }
            else
            {
                if (payment.AmountInvoice > 0 || payment.AmountUsed > 0)//如果要拆分的到账已经开过票 就要计算拆分项的开票金额
                {
                    if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Payment>();
                    //查看已经拆分的项目分得的已经开票金额 
                    var existAssigns = conn.Select<Payment>(o => o.RootKvid == payment.RootKvid && o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.Assign));
                    if (payment.AmountInvoice < payment.Amount)//到账的部分进行了开票 就需要计算拆分项的开票金额
                    {
                        //剩余的未开票金额
                        var paymentAmountUnInvoice = payment.Amount - payment.AmountInvoice;
                        if (!existAssigns.IsNullOrEmpty())
                        {
                            //已经划转未开票的金额
                            var existAssignsAmountUnInvoice = existAssigns.Sum(o => o.Amount) - existAssigns.Sum(o => o.AmountInvoice);

                            var leftAmountUnInvoice = paymentAmountUnInvoice - existAssignsAmountUnInvoice < 0 ? 0 : paymentAmountUnInvoice - existAssignsAmountUnInvoice;

                            //要是剩余未开票金额小于要划转金额那么划转开票金额就是差值否则就是0元
                            amountInvoice = leftAmountUnInvoice < amountAssign ? amountAssign - leftAmountUnInvoice : 0;
                        }
                        else
                        {
                            amountInvoice = paymentAmountUnInvoice < amountAssign ? amountAssign - paymentAmountUnInvoice : 0;
                        }
                    }
                    else amountInvoice = amountAssign;//否则直接记录拆分项完全开票
                    if (payment.AmountUsed < payment.Amount)
                    {
                        //剩余的未使用金额
                        var paymentAmountUnUsed = payment.Amount - payment.AmountUsed;
                        if (!existAssigns.IsNullOrEmpty())
                        {
                            //已经划转未使用的金额
                            var existAssignsAmountUnUsed = existAssigns.Sum(o => o.Amount) - existAssigns.Sum(o => o.AmountUsed);

                            var leftAmountUnUsed = paymentAmountUnUsed - existAssignsAmountUnUsed < 0 ? 0 : paymentAmountUnUsed - existAssignsAmountUnUsed;

                            //要是剩余未使用金额小于要划转金额那么划转使用金额就是差值否则就是0元
                            amountUsed = leftAmountUnUsed < amountAssign ? amountAssign - leftAmountUnUsed : 0;
                        }
                        else
                        {
                            amountUsed = paymentAmountUnUsed < amountAssign ? amountAssign - paymentAmountUnUsed : 0;
                        }
                    }
                    else amountUsed = amountAssign;
                }
            }
            #endregion

            #region 新增拆分项目
            var assignPayment = new Payment();
            assignPayment.PopulateWith(payment);//先从父级中拷贝所有参数
                                                //赋值
            assignPayment.Kvid = Guid.NewGuid();
            assignPayment.ParentKvid = payment.Kvid;
            assignPayment.RootKvid = payment.RootKvid;
            assignPayment.OffsetKvid = Guid.Empty;
            assignPayment.OwnerKvid = ownerKvid == null ? Guid.Empty : ownerKvid.Value;
            assignPayment.OwnerName = ownerName;
            //业务设置
            assignPayment.BizId = string.Empty;
            assignPayment.BizKvid = Guid.Empty;
            assignPayment.BizType = string.Empty;
            assignPayment.SerialNumber = payment.GetSubSerialNumber();
            assignPayment.Type = PaymentType.Assign;
            //金额设置
            assignPayment.AuditorName = assignPayment.OperateTime.Year.ToString();
            assignPayment.Amount = amountAssign;
            assignPayment.AmountSplited = 0;
            assignPayment.AmountUsed = amountUsed;
            assignPayment.AmountInvoice = amountInvoice;// payment.AmountInvoice > amountSplit ? amountSplit : payment.AmountInvoice;
            //splitPayment.Summary = string.Empty;
            if (!remark.IsNullOrEmpty()) assignPayment.Remark += $"[划转备注: {remark}]";
            assignPayment.OperateTime = payment.OperateTime;
            assignPayment.OperatorName = KiviiContext.CurrentMember.FullName;
            assignPayment.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            if (assignPayment.Metadata.IsNullOrEmpty()) assignPayment.Metadata = new Dictionary<string, string>();
            assignPayment.Metadata["AssignOwnerName"] = payment.OwnerName;
            assignPayment.Metadata["AssignOwnerKvid"] = payment.OwnerKvid.ToString();
            //splitPayment.Status = 1;
            #endregion

            return assignPayment;
        }

        /// <summary>
        /// complate!
        /// 到账拆分,并且更新原payment.AmountSplit字段
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="amountSplit"></param>
        /// <param name="ownerKvid"></param>
        /// <param name="ownerName"></param>
        /// <returns></returns>
        public static Payment BizSplit(this Payment payment, decimal amountSplit, string remark = null, Guid? ownerKvid = null, string ownerName = null, IDbConnection conn = null)
        {
            payment.ThrowIfNull($"传入payment参数为空");
            if (payment.Type != PaymentType.Bank) throw new Exception("非银行到账不可以认领!");
            if (payment.OffsetKvid != Guid.Empty) throw new Exception("已冲销收款不可认领!");
            (amountSplit <= 0).ThrowIfTrue("拆分认领金额不能为零或负值。");
            if (payment.AmountSplited == payment.Amount) throw new Exception("无剩余收款可认领!");
            if (payment.AmountSplited + amountSplit > payment.Amount) throw new Exception("认领金额超出范围!");
            decimal amountInvoice = 0;
            if (payment.AmountInvoice > 0)//如果要拆分的到账已经开过票 就要计算拆分项的开票金额
            {
                if (payment.AmountInvoice < payment.Amount)//到账的部分进行了开票 就需要计算拆分项的开票金额
                {
                    if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Payment>();
                    //查看已经拆分的项目分得的已经开票金额 
                    var existSplits = conn.Select<Payment>(o => o.RootKvid == payment.RootKvid && o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.Split, PaymentType.UnBiz, PaymentType.Refund));
                    if (!existSplits.IsNullOrEmpty())
                    {
                        //剩余的开票金额
                        var remainAmountInvoice = payment.AmountInvoice - existSplits.Sum(o => o.AmountInvoice);
                        if (remainAmountInvoice > 0) amountInvoice = remainAmountInvoice > amountSplit ? amountSplit : remainAmountInvoice;
                    }
                    else
                    {
                        amountInvoice = payment.AmountInvoice > amountSplit ? amountSplit : payment.AmountInvoice;
                    }
                }
                else amountInvoice = amountSplit;//否则直接记录拆分项完全开票
            }
            #region 新增拆分项目
            var splitPayment = new Payment();
            splitPayment.PopulateWith(payment);//先从父级中拷贝所有参数
                                               //赋值
            splitPayment.Kvid = Guid.NewGuid();
            splitPayment.ParentKvid = payment.Kvid;
            splitPayment.RootKvid = payment.RootKvid;
            splitPayment.OffsetKvid = Guid.Empty;
            splitPayment.OwnerKvid = ownerKvid == null ? Guid.Empty : ownerKvid.Value;
            splitPayment.OwnerName = ownerName;
            //业务设置
            splitPayment.BizId = string.Empty;
            splitPayment.BizKvid = Guid.Empty;
            splitPayment.BizType = string.Empty;
            splitPayment.SerialNumber = payment.GetSubSerialNumber();
            splitPayment.Type = PaymentType.Split;
            //金额设置
            splitPayment.AuditorName = splitPayment.OperateTime.Year.ToString();
            splitPayment.Amount = amountSplit;
            splitPayment.AmountSplited = 0;
            splitPayment.AmountUsed = 0;
            splitPayment.AmountInvoice = amountInvoice;// payment.AmountInvoice > amountSplit ? amountSplit : payment.AmountInvoice;
            //splitPayment.Summary = string.Empty;
            if (!remark.IsNullOrEmpty()) splitPayment.Remark += $"[认领备注: {remark}]";
            splitPayment.OperateTime = payment.OperateTime;
            splitPayment.OperatorName = KiviiContext.CurrentMember.FullName;
            splitPayment.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            //splitPayment.Status = 1;
            #endregion

            return splitPayment;
        }

        /// <summary>
        /// complate!
        /// 非业务收款拆分处理,非业务收款或退款
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="amountSplit"></param>
        /// <param name="ownerKvid"></param>
        /// <param name="ownerName"></param>
        /// <returns></returns>
        public static Payment UnBizSplit(this Payment payment, decimal amountSplit, PaymentType paymentType, string remark = null, IDbConnection conn = null)
        {
            payment.ThrowIfNull($"传入payment参数为空");
            if (payment.Type != PaymentType.Bank) throw new Exception("非银行到账不可以操作!");
            if (payment.OffsetKvid != Guid.Empty) throw new Exception("已冲销收款不可操作!");
            (amountSplit <= 0).ThrowIfTrue("操作金额不能为零或负值。");
            if (payment.AmountSplited == payment.Amount) throw new Exception("无剩余收款可操作!");
            if (payment.AmountSplited + amountSplit > payment.Amount) throw new Exception("操作金额超出范围!");
            if (paymentType != PaymentType.UnBiz && paymentType != PaymentType.Refund) throw new Exception("此方法只支持非业务拆分或退款拆分!");
            decimal amountInvoice = 0;
            if (payment.AmountInvoice > 0)//如果要拆分的到账已经开过票 就要计算拆分项的开票金额
            {
                if (payment.AmountInvoice < payment.Amount)//到账的部分进行了开票 就需要计算拆分项的开票金额
                {
                    if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Payment>();
                    //查看已经拆分的项目分得的已经开票金额 
                    var existSplits = conn.Select<Payment>(o => o.RootKvid == payment.RootKvid && o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.Split, PaymentType.UnBiz, PaymentType.Refund));
                    if (!existSplits.IsNullOrEmpty())
                    {
                        //剩余的开票金额
                        var remainAmountInvoice = payment.AmountInvoice - existSplits.Sum(o => o.AmountInvoice);
                        if (remainAmountInvoice > 0) amountInvoice = remainAmountInvoice > amountSplit ? amountSplit : remainAmountInvoice;
                    }
                    else
                    {
                        amountInvoice = payment.AmountInvoice > amountSplit ? amountSplit : payment.AmountInvoice;
                    }
                }
                else amountInvoice = amountSplit;//否则直接记录拆分项完全开票
            }
            #region 新增拆分项目
            var splitPayment = new Payment();
            splitPayment.PopulateWith(payment);//先从父级中拷贝所有参数
                                               //赋值
            splitPayment.Kvid = Guid.NewGuid();
            splitPayment.ParentKvid = payment.Kvid;
            splitPayment.RootKvid = payment.RootKvid;
            splitPayment.OffsetKvid = Guid.Empty;
            splitPayment.OwnerKvid = Guid.Empty;
            splitPayment.OwnerName = string.Empty;
            //业务设置
            splitPayment.BizId = string.Empty;
            splitPayment.BizKvid = Guid.Empty;
            splitPayment.BizType = string.Empty;
            splitPayment.SerialNumber = payment.GetSubSerialNumber();
            splitPayment.Type = paymentType;
            //金额设置
            splitPayment.AuditorName = splitPayment.OperateTime.Year.ToString();
            splitPayment.Amount = amountSplit;
            splitPayment.AmountSplited = 0;
            splitPayment.AmountUsed = 0;
            splitPayment.AmountInvoice = amountInvoice;
            splitPayment.Summary = string.Empty;
            splitPayment.Remark = remark;
            splitPayment.OperateTime = payment.OperateTime;
            splitPayment.OperatorName = KiviiContext.CurrentMember.FullName;
            splitPayment.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            //splitPayment.Status = 1;
            #endregion

            return splitPayment;
        }

    }
}
