﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{

    [Api(Description = "今日流水操作统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentAcceptStatistic : RestfulExecution<Payment>
    {
        public DateTime BeginTime { get; set; }//日期筛选为创建日期,默认统计今日导入系统中的流水信息

        public DateTime EndTime { get; set; }


        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = DateTime.Now;
            if (EndTime == DateTime.MinValue) EndTime = DateTime.Now;

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);

            var rtns = new StatisticResponse<Analysis>();

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var queryAccount = conn.From<Account>();
            queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
            queryAccount.Select(o => o.Kvid);

            var sqlExpress = conn.From<Payment>();
            sqlExpress.Where(o => o.RootKvid == o.Kvid && o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.AliPay, PaymentType.WeChat, PaymentType.Bank, PaymentType.Cash, PaymentType.Pos));
            sqlExpress.And(o => o.CreateTime >= beginTime && o.CreateTime < endTime);
            //sqlExpress.And(o => o.AmountSplited == 0 || o.AmountSplited < o.Amount);
            sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
            //if (IsTest)
            //{

            sqlExpress.Select(o => new
            {
                Total = Sql.Count(o.Kvid),
                TotalAmount = Sql.Sum(o.Amount),
                TotalAmountSplited = Sql.Sum(o.Amount - o.AmountSplited),
                TotalAmountInvoice = Sql.Sum(o.Amount - o.AmountInvoice),
                TotalAmountUsed = Sql.Sum(o.Amount - o.AmountUsed),
                TotalQuantitySplited = Sql.Sum((o.AmountSplited == 0 || o.AmountSplited < o.Amount) ? 1 : 0),
                TotalQuantityInvoice = Sql.Sum((o.AmountInvoice == 0 || o.AmountInvoice < o.Amount) ? 1 : 0),
                TotalQuantityUsed = Sql.Sum((o.AmountUsed == 0 || o.AmountUsed < o.Amount) ? 1 : 0)

            });
            rtns = conn.Single<StatisticResponse<Analysis>>(sqlExpress);
            rtns.Results = new List<Analysis>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            return rtns;
            //}
            //sqlExpress.Select(o => new { o.AmountSplited, o.AmountInvoice, o.AmountUsed, o.Amount });
            //var results = conn.Select(sqlExpress);

            //rtns.Results = new List<Analysis>();
            ////rtns.Description = "待认领流水";
            //rtns.BeginTime = BeginTime;
            //rtns.EndTime = EndTime;
            //if (results.IsNullOrEmpty()) return rtns;
            //rtns.Total = results.Count();
            //rtns.TotalAmount = results.Sum(o => o.Amount);
            //rtns.TotalAmountSplited = results.Sum(o => o.Amount - o.AmountSplited);
            //rtns.TotalQuantitySplited = results.Where(o => o.AmountSplited == 0 || o.AmountSplited < o.Amount).Count();
            //rtns.TotalAmountInvoice = results.Sum(o => o.Amount - o.AmountInvoice);
            //rtns.TotalQuantityInvoice = results.Where(o => o.AmountInvoice == 0 || o.AmountInvoice < o.Amount).Count();
            //rtns.TotalAmountUsed = results.Sum(o => o.Amount - o.AmountUsed);
            //rtns.TotalQuantityUsed = results.Where(o => o.AmountUsed == 0 || o.AmountUsed < o.Amount).Count();
            //return rtns;
        }
    }

    [Api(Description = "已认领流水分析")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentSplitedAnalysis : RestfulExecution<Payment>
    {
        public Guid OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }

        public bool GroupByOwner { get; set; }

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public bool IsOperated { get; set; }//如果True则查询OperateTime否则默认CreateTime
        //public bool IsTest { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var queryAccount = conn.From<Account>();
            queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
            queryAccount.Select(o => o.Kvid);

            var sqlExpress = conn.From<Payment>();
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.WeChat, PaymentType.Cash, PaymentType.AliPay, PaymentType.Pos, PaymentType.Split));
            if (IsOperated) sqlExpress.And(o => o.OperateTime >= beginTime && o.OperateTime < endTime);
            else sqlExpress.And(o => o.CreateTime >= beginTime && o.CreateTime < endTime);
            if (!OwnerKvids.IsNullOrEmpty()) sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
            sqlExpress.OrderBy(o => o.OwnerName);
            //sqlExpress.Select(o => new { o.OwnerKvid, o.OwnerName, o.Amount, o.AmountInvoice, o.AmountUsed, o.OperateTime, o.CreateTime, o.Type });
            //var results = conn.Select(sqlExpress);
            var rtns = new StatisticResponse<Analysis>();
            //if (results.IsNullOrEmpty()) return rtns;
            rtns.Results = new List<Analysis>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;

            //传入部门信息就按照部门group
            if (GroupByOwner)
            {
                sqlExpress.GroupBy(o => o.OwnerName);
                sqlExpress.Select(o => new
                {
                    Summary = o.OwnerName,
                    Quantity = Sql.Count(o.Kvid),
                    Amount = Sql.Sum(o.Amount),
                    AmountSplit = Sql.Sum(o.Amount),
                    QuantitySplit = Sql.Count(o.Kvid),
                    AmountInvoice = Sql.Sum(o.AmountInvoice),
                    QuantityInvoice = Sql.Sum(o.Amount == o.AmountInvoice ? 1 : 0),
                    AmountNotInvoice = Sql.Sum(o.Amount - o.AmountInvoice),
                    QuantityNotInvoice = Sql.Sum(o.AmountInvoice < o.Amount ? 1 : 0),
                    AmountUsed = Sql.Sum(o.AmountUsed),
                    QuantityUsed = Sql.Sum(o.AmountUsed == o.Amount ? 1 : 0),
                    AmountNotUsed = Sql.Sum(o.Amount - o.AmountUsed),
                    QuantityNotUsed = Sql.Sum(o.AmountUsed < o.Amount ? 1 : 0)
                });
                rtns.Results = conn.Select<Analysis>(sqlExpress);
                //var group = results.GroupBy(o => o.OwnerName);
                //foreach (var kv in group)
                //{
                //    var rtn = new Analysis();
                //    rtn.Summary = kv.Key;
                //    rtn.Amount = kv.Sum(o => o.Amount);
                //    rtn.Quantity = kv.Count();
                //    rtn.AmountSplit = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.Amount);
                //    rtn.QuantitySplit = kv.Count(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos);
                //    rtn.AmountInvoice = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.AmountInvoice);
                //    rtn.QuantityInvoice = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Count(o => o.Amount == o.AmountInvoice);
                //    rtn.AmountNotInvoice = rtn.AmountSplit - rtn.AmountInvoice;
                //    rtn.QuantityNotInvoice = rtn.QuantitySplit - rtn.QuantityInvoice;
                //    rtn.AmountUsed = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.AmountUsed);
                //    rtn.QuantityUsed = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Count(o => o.Amount == o.AmountUsed);
                //    rtn.AmountNotUsed = rtn.AmountSplit - rtn.AmountUsed;
                //    rtn.QuantityNotUsed = rtn.QuantitySplit - rtn.QuantityUsed;
                //    rtn.AmountRefund = kv.Where(o => o.Type == PaymentType.Refund).Sum(o => o.Amount);
                //    rtn.QuantityRefund = kv.Count(o => o.Type == PaymentType.Refund);
                //    rtn.AmountUnBiz = kv.Where(o => o.Type == PaymentType.UnBiz).Sum(o => o.Amount);
                //    rtn.QuantityUnBiz = kv.Count(o => o.Type == PaymentType.UnBiz);
                //    rtns.Results.Add(rtn);
                //}
            }
            else
            {
                sqlExpress.Select(o => new
                {
                    TotalQuantity = Sql.Count(o.Kvid),
                    TotalAmount = Sql.Sum(o.Amount),
                    TotalAmountSplited = Sql.Sum(o.Amount),
                    TotalQuantitySplited = Sql.Count(o.Kvid),
                    TotalAmountInvoice = Sql.Sum(o.AmountInvoice),
                    TotalQuantityInvoice = Sql.Sum(o.Amount == o.AmountInvoice ? 1 : 0),
                    TotalAmountNotInvoice = Sql.Sum(o.Amount - o.AmountInvoice),
                    TotalQuantityNotInvoice = Sql.Sum(o.AmountInvoice < o.Amount ? 1 : 0),
                    TotalAmountUsed = Sql.Sum(o.AmountUsed),
                    TotalQuantityUsed = Sql.Sum(o.AmountUsed == o.Amount ? 1 : 0),
                    TotalAmountNotUsed = Sql.Sum(o.Amount - o.AmountUsed),
                    TotalQuantityNotUsed = Sql.Sum(o.AmountUsed < o.Amount ? 1 : 0)
                });
                rtns = conn.Single<StatisticResponse<Analysis>>(sqlExpress);
                rtns.Results = new List<Analysis>();
                rtns.BeginTime = BeginTime;
                rtns.EndTime = EndTime;
                //rtns.TotalAmount = results.Sum(o => o.Amount);
                //rtns.TotalQuantity = results.Count();
                //rtns.TotalAmountSplited = results.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.Amount);
                //rtns.TotalQuantitySplited = results.Count(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos);
                //rtns.TotalAmountInvoice = results.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.AmountInvoice);
                //rtns.TotalQuantityInvoice = results.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Count(o => o.Amount == o.AmountInvoice);
                //rtns.TotalAmountNotInvoice = rtns.TotalAmountSplited - rtns.TotalAmountInvoice;
                //rtns.TotalQuantityNotInvoice = rtns.TotalQuantitySplited - rtns.TotalQuantityInvoice;
                //rtns.TotalAmountUsed = results.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.AmountUsed);
                //rtns.TotalQuantityUsed = results.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Count(o => o.Amount == o.AmountUsed);
                //rtns.TotalAmountNotUsed = rtns.TotalAmountSplited - rtns.TotalAmountUsed;
                //rtns.TotalQuantityNotUsed = rtns.TotalQuantitySplited - rtns.TotalQuantityUsed;
                //rtns.TotalAmountRefund = results.Where(o => o.Type == PaymentType.Refund).Sum(o => o.Amount);
                //rtns.TotalQuantityRefund = results.Count(o => o.Type == PaymentType.Refund);
                //rtns.TotalAmountUnBiz = results.Where(o => o.Type == PaymentType.UnBiz).Sum(o => o.Amount);
                //rtns.TotalQuantityUnBiz = results.Count(o => o.Type == PaymentType.UnBiz);
            }
            return rtns;
        }
    }

    [Api(Description = "到账流水无操作分析")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentNoOperateAnalysis : RestfulExecution<Payment>
    {
        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);
            var rtns = new StatisticResponse<Analysis>();

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var queryAccount = conn.From<Account>();
            queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
            queryAccount.Select(o => o.Kvid);

            var sqlExpress = conn.From<Payment>();
            sqlExpress.Where(o => o.RootKvid == o.Kvid && o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.AliPay, PaymentType.WeChat, PaymentType.Bank, PaymentType.Cash, PaymentType.Pos));
            sqlExpress.And(o => o.OperateTime >= beginTime && o.OperateTime < endTime);
            //sqlExpress.And(o => o.AmountSplited == 0 || o.AmountSplited < o.Amount);
            sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
            sqlExpress.Select(o => new
            {
                Total = Sql.Count(o.Kvid),
                TotalQuantity = Sql.Count(o.Kvid),
                TotalAmount = Sql.Sum(o.Amount),
                TotalAmountNotSplited = Sql.Sum(o.Amount - o.AmountSplited),
                TotalQuantityNotSplited = Sql.Sum(o.AmountSplited < o.Amount ? 1 : 0),
                TotalAmountNotInvoice = Sql.Sum(o.Amount - o.AmountInvoice),
                TotalQuantityNotInvoice = Sql.Sum(o.AmountInvoice < o.Amount ? 1 : 0),
                TotalAmountNotUsed = Sql.Sum(o.Amount - o.AmountUsed),
                TotalQuantityNotUsed = Sql.Sum(o.AmountUsed < o.Amount ? 1 : 0)
            });
            rtns = conn.Single<StatisticResponse<Analysis>>(sqlExpress);
            rtns.Results = new List<Analysis>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;

            return rtns;

        }
    }

    [Api(Description = "部门流水划拨分析")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentOwneSplitedAnalysis : RestfulExecution<Payment>
    {
        public Guid OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public bool IsDetail { get; set; }//如果是查询明细,就按照部门把划拨到各个部门的流水列出来,如果不是查明细就查当前部门流水的划拨情况,划拨了多少,待划拨多少

        public override object OnExecution(IRequest req, IResponse res)
        {
            //(OwnerKvid == Guid.Empty).ThrowIfTrue("请传入部门信息以统计划拨统计");
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);
            var rtns = new StatisticResponse<Analysis>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids.AddRange(KiviiContext.CurrentMember.DepartmentKvids);
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();

            var queryAccount = conn.From<Account>();
            queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
            queryAccount.Select(o => o.Kvid);

            var queryRoot = conn.From<Payment>();
            queryRoot.Where(o => o.OffsetKvid == Guid.Empty && o.Type == PaymentType.Bank && Sql.In(o.OwnerKvid, OwnerKvids) && Sql.In(o.PayerAccountKvid, queryAccount) && o.OperateTime >= beginTime && o.OperateTime < endTime);

            //如果是查询明细,就按照部门把划拨到各个部门的流水列出来,如果不是查明细就查当前部门流水的划拨情况,划拨了多少,待划拨多少
            if (IsDetail)
            {
                queryRoot.Select(o => o.Kvid);
                var query = conn.From<Payment>();
                query.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.Refund, PaymentType.UnBiz, PaymentType.Split) && Sql.In(o.ParentKvid, queryRoot));
                query.GroupBy(o => new { o.OwnerName, o.Type });
                query.Select(o => new
                {
                    Summary = o.OwnerName,
                    Type = o.Type,
                    Quantity = Sql.Count(o.Kvid),
                    Amount = Sql.Sum(o.Amount),
                    AmountInvoice = Sql.Sum(o.AmountInvoice),
                    QuantityInvoice = Sql.Sum(o.Amount == o.AmountInvoice ? 1 : 0),
                    AmountNotInvoice = Sql.Sum(o.Amount - o.AmountInvoice),
                    QuantityNotInvoice = Sql.Sum(o.AmountInvoice < o.Amount ? 1 : 0),
                    AmountUsed = Sql.Sum(o.AmountUsed),
                    QuantityUsed = Sql.Sum(o.Amount == o.AmountUsed ? 1 : 0),
                    AmountNotUsed = Sql.Sum(o.Amount - o.AmountUsed),
                    QuantityNotUsed = Sql.Sum(o.AmountUsed < o.Amount ? 1 : 0)
                });
                rtns.Results = conn.Select<Analysis>(query);
                if (!rtns.Results.IsNullOrEmpty())
                {
                    rtns.TotalAmount = rtns.Results.Sum(o => o.Amount);
                    rtns.TotalAmountInvoice = rtns.Results.Sum(o => o.AmountInvoice);
                    rtns.TotalAmountNotInvoice = rtns.Results.Sum(o => o.AmountNotInvoice);
                    rtns.TotalAmountUsed = rtns.Results.Sum(o => o.AmountUsed);
                    rtns.TotalAmountNotUsed = rtns.Results.Sum(o => o.AmountNotUsed);
                    rtns.TotalQuantity = rtns.Results.Sum(o => o.Quantity);
                    rtns.TotalQuantityInvoice = rtns.Results.Sum(o => o.QuantityInvoice);
                    rtns.TotalQuantityNotInvoice = rtns.Results.Sum(o => o.QuantityNotInvoice);
                    rtns.TotalQuantityUsed = rtns.Results.Sum(o => o.QuantityUsed);
                    rtns.TotalQuantityNotUsed = rtns.Results.Sum(o => o.QuantityNotUsed);
                }
            }
            else
            {
                queryRoot.GroupBy(o => o.OwnerName);
                queryRoot.Select(o => new
                {
                    Summary = o.OwnerName,
                    Quantity = Sql.Count(o.Kvid),
                    Amount = Sql.Sum(o.Amount),
                    AmountSplit = Sql.Sum(o.AmountSplited),
                    QuantitySplit = Sql.Sum(o.Amount == o.AmountSplited ? 1 : 0),
                    AmountNotSplit = Sql.Sum(o.Amount - o.AmountSplited),
                    QuantityNotSplit = Sql.Sum(o.AmountSplited < o.Amount ? 1 : 0),
                    AmountInvoice = Sql.Sum(o.AmountInvoice),
                    QuantityInvoice = Sql.Sum(o.Amount == o.AmountInvoice ? 1 : 0),
                    AmountNotInvoice = Sql.Sum(o.Amount - o.AmountInvoice),
                    QuantityNotInvoice = Sql.Sum(o.AmountInvoice < o.Amount ? 1 : 0),
                    AmountUsed = Sql.Sum(o.AmountUsed),
                    QuantityUsed = Sql.Sum(o.Amount == o.AmountUsed ? 1 : 0),
                    AmountNotUsed = Sql.Sum(o.Amount - o.AmountUsed),
                    QuantityNotUsed = Sql.Sum(o.AmountUsed < o.Amount ? 1 : 0)
                });
                rtns.Results = conn.Select<Analysis>(queryRoot);
                if (!rtns.Results.IsNullOrEmpty())
                {
                    rtns.TotalAmount = rtns.Results.Sum(o => o.Amount);
                    rtns.TotalAmountSplited = rtns.Results.Sum(o => o.AmountSplit);
                    rtns.TotalAmountNotSplited = rtns.Results.Sum(o => o.AmountNotSplit);
                    rtns.TotalAmountInvoice = rtns.Results.Sum(o => o.AmountInvoice);
                    rtns.TotalAmountNotInvoice = rtns.Results.Sum(o => o.AmountNotInvoice);
                    rtns.TotalAmountUsed = rtns.Results.Sum(o => o.AmountUsed);
                    rtns.TotalAmountNotUsed = rtns.Results.Sum(o => o.AmountNotUsed);
                    rtns.TotalQuantity = rtns.Results.Sum(o => o.Quantity);
                    rtns.TotalQuantityNotSplited = rtns.Results.Sum(o => o.QuantityNotSplit);
                    rtns.TotalQuantityInvoice = rtns.Results.Sum(o => o.QuantityInvoice);
                    rtns.TotalQuantityNotInvoice = rtns.Results.Sum(o => o.QuantityNotInvoice);
                    rtns.TotalQuantityUsed = rtns.Results.Sum(o => o.QuantityUsed);
                    rtns.TotalQuantityNotUsed = rtns.Results.Sum(o => o.QuantityNotUsed);
                }
            }

            rtns.Total = rtns.Results.Count();
            return rtns;
        }
    }

    [Api(Description = "部门收入分析")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentGroupYearAnalysis : RestfulExecution<Payment>
    {
        public string Year { get; set; }

        public Guid OwnerKvid { get; set; }

        public List<Guid> OwnerKvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            Year.ThrowIfNullOrEmpty("请传入收入年份");
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var query = conn.From<Payment>();
            query.Where(o => o.AuditorName == Year && o.OffsetKvid == Guid.Empty && o.Type == PaymentType.Split);
            if (!OwnerKvids.IsNullOrEmpty()) query.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            query.GroupBy(o => new { o.OwnerName, o.AuditorName });
            query.Select(o => new
            {
                AuditorName = o.AuditorName,
                OwnerName = o.OwnerName,
                Amount=Sql.Sum(o.Amount)
            });
            var rtns = new RestfulQueryResponse<Payment>();
            rtns.Results = conn.Select(query);
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
}
